

local TagCompiler = AceLibrary("TagCompiler-1.0")

-----------------------------------------------------------------------------

Assessment.TagData = 
{ 
	TagError = function(x) Assessment:Print(x) end, 
--	TagDebug = function(x) Assessment:Debug(x) end 
}

-----------------------------------------------------------------------------

for k,v in pairs(TagCompiler.CommonTagData) do
	Assessment.TagData[k] = v
end

-----------------------------------------------------------------------------

Assessment.TagData.amount =
{
	tableData = "value.amount",
	numeric = true
}

Assessment.TagData.barvalue =
{
	tableData = "barValue",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.count =
{
	tableData = "value.count",
	numeric = true
}

Assessment.TagData.formattime =
{
	method = function(data, value)
				if type(value) == "number" then
					return date("%H:%M:%S", value)
				end
				return value
			 end,
	noGuaranteeChange = true
}

Assessment.TagData.formatduration =
{
	method = function(data, value)
				if type(value) == "number" then
					if value < 60 then
						return ("%.1f"):format(value)
					else
						return ("%d:%02d.%d"):format(value/60, value%60, value%1*10)
					end
				end
				return value
			 end,
	noGuaranteeChange = true
}

Assessment.TagData.isaltkeydown =
{
	compileString = function() return "IsAltKeyDown()" end,
	events = "MODIFIER_STATE_CHANGED",
}

Assessment.TagData.iscontrolkeydown =
{
	compileString = function() return "IsControlKeyDown()" end,
	events = "MODIFIER_STATE_CHANGED",
}

Assessment.TagData.isplayerincombat =
{
	compileString = function() return "UnitAffectingCombat(\"player\")" end,
	events = { "PLAYER_REGEN_ENABLED", "PLAYER_REGEN_DISABLED" }
}

Assessment.TagData.isreport =
{
	tableData = "isReport",
}

Assessment.TagData.isshiftkeydown =
{
	compileString = function() return "IsShiftKeyDown()" end,
	events = "MODIFIER_STATE_CHANGED",
}

Assessment.TagData.key =
{
	tableData = "key",
}

Assessment.TagData.maximumbarvalue =
{
	tableData = "maximumBarValue",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.rank =
{
	tableData = "rank",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.sortValue =
{
	tableData = "sortValue",
	guaranteeResult = true,
}

Assessment.TagData.totalbarvalue =
{
	tableData = "totalBarValue",
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.value =
{
	tableData = "value",
}

-----------------------------------------------------------------------------

local function GetField(data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return end
		data = data[select(i, ...)]
	end
	return data
end

local function SumField(data, ...)
	for i=1,select('#', ...) do
		if type(data) ~= "table" then return 0 end
		local lookup = select(i, ...)
		if lookup == "*" then
			local sum = 0
			for k,v in pairs(data) do
				sum = sum + SumField(v, select(i+1, ...))
			end
			return sum
		else
			data = data[lookup]
		end
	end
	return data or 0
end

Assessment.TagData.getfield =
{
	method = function(data, value, lookup)
				return GetField(data.fightData, ("."):split(lookup))
			 end,
	parameterTypes = { "string" }
}

Assessment.TagData.getvaluefield =
{
	method = function(data, value, lookup)
				return GetField(data.value, ("."):split(lookup))
			 end,
	parameterTypes = { "string" }
}

Assessment.TagData.sumfield =
{
	method = function(data, value, lookup)
				return SumField(data.fightData, ("."):split(lookup))
			 end,
	parameterTypes = { "string" },
	guaranteeResult = true,
	numeric = true
}

Assessment.TagData.sumvaluefield =
{
	method = function(data, value, lookup)
				return SumField(data.value, ("."):split(lookup))
			 end,
	parameterTypes = { "string" },
	guaranteeResult = true,
	numeric = true
}

-----------------------------------------------------------------------------

local compileCache =
{
	["string"] = { data = { }, events = { }, tag = "", method = function() return "" end },
}

function Assessment:CompileTag(tag, resultType)
	tag = tag or ""
	local cacheTag = resultType..tag
	if compileCache[cacheTag] then
		return compileCache[cacheTag]
	end

	local result = TagCompiler:Compile(tag, self.TagData, resultType)
	compileCache[cacheTag] = result
	return result
end

-----------------------------------------------------------------------------
