-----------------------------------------------------------------------------
--
-- data
--   \deaths
--     \<deathName>
--       - count
--       \deathTimes (Array)
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentDeaths")
L:RegisterTranslations("enUS", function() return {
	["Deaths List"] = true,
	["Deaths"] = true,
	["Death Times"] = true,
	["Kill List"] = true,
	["Kill Times"] = true,

	["Death"] = true,
	["Death Options."] = true,
	["Group Only"] = true,
	["Only record deaths caused by group members."] = true,
} end)

-----------------------------------------------------------------------------

local AssessmentDeaths = Assessment:NewModule("Deaths", "AceEvent-2.0", "Parser-3.0")
AssessmentDeaths.db = Assessment:AcquireDBNamespace("Deaths")
Assessment:RegisterDefaults("Deaths", "profile", {
	enable				= true,
	gatherGroupOnly		= true
})

local profile

-----------------------------------------------------------------------------

Assessment.TagData.deaths =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local deaths = fightData.deaths
				if not deaths then return 0 end
				
				if not deaths[data.key] then return 0 end
				return deaths[data.key].count
			 end,
	data = "deaths",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.DeathsList =
{
	method = function(fightData)
				if not fightData.deaths then return function() return nil end end
				return pairs(fightData.deaths)
			 end,
	data = "deaths",
	description = L["Deaths List"],
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.deaths = "deaths"

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.DeathTimes =
{
	titleTag			= "[windowParameter(1)]",
	listType			= "EnumerateTable", 
	listDataTag			= "deaths,[windowParameter(1)],deathTimes",
	barValueTag			= "0", 
	filterTag			= "1",
	fightIndex			= 0,
	text				= { count = 2, { tag= "Death [rank]" }, { tag = "[value:formatTime]" } },
	description			= L["Death Times"],
	selectable			= false
}

Assessment.DefaultDisplaySets.Deaths =
{
	titleTag			= "Deaths",
	listType			= "GroupMembers", 
	barValueTag			= "deaths", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 0,
	description			= L["Deaths"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Death Times"],
	onClickData			= "[key]",
	favorite			= false,
	colorTypeList		= { "Class", count = 1 },
}

Assessment.DefaultDisplaySets.KillTimes =
{
	titleTag			= "[windowParameter(1)]",
	listType			= "EnumerateTable", 
	listDataTag			= "deaths,[windowParameter(1)],deathTimes",
	barValueTag			= "0", 
	filterTag			= "1",
	fightIndex			= 0,
	text				= { count = 2, { tag= "Kill [rank]" }, { tag = "[value:formatTime]" } },
	description			= L["Kill Times"],
	selectable			= false
}

Assessment.DefaultDisplaySets.KillList =
{
	titleTag			= "Kill List",
	listType			= "DeathsList", 
	barValueTag			= "deaths", 
	filterTag			= "~isGroupMember",
	fightIndex			= 0,
	description			= L["Kill List"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Kill Times"],
	onClickData			= "[key]",
	colorTypeList		= { "Class", count = 1 },
}

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.death =
{
	type = "group",
	name = L["Death"],
	desc = L["Death Options."],
	args =
	{
		groupOnly =
		{
			type = "toggle",
			name = L["Group Only"],
			desc = L["Only record deaths caused by group members."],
			get = function() return profile.gatherGroupOnly end,
			set = function(v) profile.gatherGroupOnly = v end,
		},
	}
}

-----------------------------------------------------------------------------

function AssessmentDeaths:OnInitialize()
	profile = self.db.profile
end

function AssessmentDeaths:OnEnable()
	self:RegisterParserEvent({ eventType = "Death" }, "OnParserDeathEvent")
end

function AssessmentDeaths:OnDisable()
end

-----------------------------------------------------------------------------

function AssessmentDeaths:OnParserDeathEvent(eventData)
	if eventData.recipientID then
		-- It's a group member that died.. record it.
	else
		-- A non-group member..
		
		-- If the killer is not known, then return now
		if not eventData.sourceName then return end

		-- If the options say to only gather kills done by group members, check that.
		if profile.gatherGroupOnly and not eventData.sourceID then return end
	end
	

	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.deaths = GetTime()
		local deaths = data.deaths
		if not deaths then
			deaths = self:NewTable()
			data.deaths = deaths
		end

		local recipientDeath = deaths[eventData.recipientName]
		if recipientDeath then
			recipientDeath.count = recipientDeath.count + 1
		else
			recipientDeath = self:NewTable()
			recipientDeath.count = 1
			recipientDeath.deathTimes = self:NewTable()
			deaths[eventData.recipientName] = recipientDeath
		end
		table.insert(recipientDeath.deathTimes, time())
	end
end

-----------------------------------------------------------------------------
